"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("underscore");
var Browser = require("zombie");
var CancellationToken_1 = require("./CancellationToken");
var enforce_1 = require("./enforce");
function init(pumlhorse) {
    pumlhorse.module('pumlhorse-browser')
        .injector('$browser', getBrowser)
        .function('setBaseUrl', setBaseUrl)
        .function('setProxy', setProxy)
        .function('open', open)
        .function('click', click)
        .function('set', setValue)
        .function('wait', wait)
        .function('check', check)
        .function('uncheck', uncheck)
        .function('choose', choose)
        .function('select', select)
        .function('unselect', unselect)
        .function('attachFile', attachFile)
        .function('find', find)
        .function('findFirst', findFirst)
        .function('getCookie', getCookie)
        .function('setCookie', setCookie)
        .function('deleteCookie', deleteCookie)
        .function('isUrl', isUrl)
        .function('_getBrowser', getBrowser);
}
exports.init = init;
function setBaseUrl(baseUrl, $browser) {
    enforce_1.default(baseUrl, 'baseUrl').isString().isNotNull();
    $browser.localhost(baseUrl);
}
function setProxy(proxyUrl, $browser) {
    enforce_1.default(proxyUrl, 'proxyUrl').isString().isNotNull();
    $browser.proxy = proxyUrl;
}
function open(url, $browser, $cancellationToken) {
    enforce_1.default(url, 'url').isString().isNotNull();
    return CancellationToken_1.CancellationToken.await($browser.visit(url), $cancellationToken);
}
function click(selector, $browser, $cancellationToken) {
    enforce_1.default(selector, 'selector').isNotNull();
    return CancellationToken_1.CancellationToken.await($browser.click(getSelectorOrElement(selector)), $cancellationToken);
}
function setValue(selector, value, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    $browser.fill(getSelectorOrElement(selector), value);
}
function check(selector, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    $browser.check(getSelectorOrElement(selector));
}
function uncheck(selector, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    $browser.uncheck(getSelectorOrElement(selector));
}
function choose(selector, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    $browser.choose(getSelectorOrElement(selector));
}
function select(selector, value, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    enforce_1.default(value, 'value').isString();
    value == null
        ? $browser.selectOption(getSelectorOrElement(selector))
        : $browser.select(getSelectorOrElement(selector), value);
}
function unselect(selector, value, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    enforce_1.default(value, 'value').isString();
    value == null
        ? $browser.unselectOption(getSelectorOrElement(selector))
        : $browser.unselect(getSelectorOrElement(selector), value);
}
function attachFile(selector, file, $browser) {
    enforce_1.default(selector, 'selector').isNotNull();
    enforce_1.default(file, 'file').isString().isNotNull();
    $browser.choose(getSelectorOrElement(selector));
}
function wait(duration, $browser, $cancellationToken) {
    return CancellationToken_1.CancellationToken.await($browser.wait(duration), $cancellationToken);
}
function find(selector, $browser) {
    enforce_1.default(selector, 'selector').isString().isNotNull();
    return $browser.queryAll(selector).map(function (elem) { return new ReadOnlyDom(elem); });
}
function findFirst(selector, $browser) {
    enforce_1.default(selector, 'selector').isString().isNotNull();
    var elem = $browser.query(selector);
    return elem == null
        ? null
        : new ReadOnlyDom(elem);
}
function getSelectorOrElement(val) {
    if (_.isString(val))
        return val;
    return val.getDomElement();
}
var ReadOnlyDom = (function () {
    function ReadOnlyDom(element) {
        this.element = element;
        this.attributes = {};
        for (var i = 0; i < element.attributes.length; i++) {
            var attr = element.attributes.item(i);
            this.attributes[attr.localName] = attr.value;
        }
        this.classList = element.classList;
        this.id = element.id;
        this.localName = element.localName;
        this.name = element.name;
        this.prefix = element.prefix;
        this.textContent = element.textContent;
        this.value = element.value;
    }
    ReadOnlyDom.prototype.getDomElement = function () {
        return this.element;
    };
    return ReadOnlyDom;
}());
function getCookie(selector, $scope) {
    enforce_1.default(selector, 'selector').isString().isNotNull();
    var browser = getBrowser($scope);
    return browser.getCookie(selector, true);
}
function setCookie(name, value, $scope) {
    enforce_1.default(name, 'name').isString().isNotNull();
    enforce_1.default(value, 'value').isNotNull();
    var browser = getBrowser($scope);
    if (_.isString(value)) {
        browser.setCookie(name, value);
    }
    else {
        var cookie = value;
        cookie.name = name;
        browser.setCookie(cookie);
    }
}
function deleteCookie(names, $scope) {
    enforce_1.default(names, 'names').isNotNull();
    var browser = getBrowser($scope);
    var cookieNames = _.isString(names)
        ? [names]
        : names;
    for (var i = 0; i < cookieNames.length; i++) {
        browser.deleteCookie(cookieNames[i]);
    }
}
function isUrl(url, $scope) {
    enforce_1.default(url, 'url').isString().isNotNull();
    var browser = getBrowser($scope);
    browser.assert.url(url);
}
function getBrowser($scope) {
    if ($scope.__zombie_browser == null) {
        $scope.__zombie_browser = new Browser();
    }
    return $scope.__zombie_browser;
}
//# sourceMappingURL=module.js.map